import os
import sys
import shutil
import re
import time
from com.ibm.websphere.management.application import *
from java.lang import System
from java.util.logging import Level
from java.util import Date
from java.util.logging import Logger

from com.ibm.wbmonitor.install.common.log import LogFactory
from com.ibm.wbmonitor.install.common.log import ILogger
from com.ibm.wbmonitor.install.common.log import LogConstants

date = Date()
f = java.text.SimpleDateFormat("yy.MM.dd_HH.mm.ss.SSS")

wasInstallRoot = java.lang.System.getProperty('was.install.root')
logDir = wasInstallRoot + '/logs/migration' 
logDirExists=os.path.exists(logDir)
if (logDirExists == 0):
    os.mkdir(logDir)

logName = "MonitorModelMigration" + f.format(date) + "_out.log"
log = LogFactory.createLogger(LogConstants.WBM_INSTALL_GROUP_NAME, 
  "MonitorModelMigration", logDir, logName)

serviceNameList = ['DSS_DMS_CALL_SP_SERVICE','DSS_KPI_HISTORY_SERVICE','DSS_KPI_PREDICTION_SERVICE','DSS_DYNAMIC_ALERT_EVAL_SERVICE','DSS_CUBE_SUMMARY_REFRESH_SERVICE','DSS_ABX_CUBE_REFRESH_SERVICE','DSS_ARCHIVE_PURGE_SERVICE','DSS_COGNOS_CACHECLEARING_TASK']

#--------------------------------------------------------------
# Helper Class to normalize the string for comparision
#--------------------------------------------------------------
def wsadminToList(inStr):
    outList=[]
    if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
        tmpList = inStr[1:-1].split() #splits space-separated lists,
    else:
        tmpList = inStr.split("\n")   #splits for Windows or Linux
    for item in tmpList:
        item = item.rstrip();         #removes any Windows "\r"
        if (len(item)>0):
            outList.append(item)
    return outList;
#endDef

def isSpecifiedScope(application,clusterName,nodeName,serverName):
    target = ''
    if(clusterName):
        target = 'cluster=' + clusterName
    else:
        target = 'node=' + nodeName + ',server='+serverName
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isSpecifiedScope','INFO: deploy target of application ' + application + ' is ' + target)
    mapModuleToServerOption = wsadminToList(AdminApp.view(application, "-MapModulesToServers"))
    pattern = re.compile("([^:]*):\s*(.*)")
    for line in mapModuleToServerOption:
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isSpecifiedScope',"DEBUG: " + "mapModuleToServerOption line:" + line)
        matched = pattern.match(line)
        if matched:       
            #if(matched.group(1).strip() == 'Server'):
                #log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isSpecifiedScope',"INFO: " + "Application's module mapped to:" + matched.group(2).strip())
            if(matched.group(2).strip().find(target) > 0):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isSpecifiedScope',"INFO: " + "Application "  + application + " is in current specified scope.")
                return 'true'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isSpecifiedScope',"INFO: " + "Application "  + application + " is not in current specified scope." )      
    return 'false'

def getAppDeploymentTaskOption(appName,optionName,blockSize):
    import AdminUtilities
    appDepRawContent = AdminUtilities.convertToList(AdminApp.view(appName, optionName))      
    appDepOptions = []

    if appDepRawContent:
      # the key/value pairs are separated by colons, and the keys may be translated    
      pattern = re.compile("[^:]*:\s*(.*)")
      # iterate all blocks (role mappings)
      for i in range(divmod(len(appDepRawContent), blockSize)[1], len(appDepRawContent), blockSize):
        # build the options list for the current block (role mapping)
        appDepOption = []
        for line in appDepRawContent[i:i + blockSize]:
          #print line
          matched = pattern.match(line)
          if matched:
            appDepOption += [matched.group(1)]
        #endfor
        appDepOptions += [appDepOption]
      #endfor
    #endif

    return appDepOptions
#enddef
'''
def isMonitorModel2(application):
    if(debug):
        print "Check if application '" + application + "' is monitor model application." 
    exportedDir = os.path.join(tempDir,application)
    if(os.path.exists(exportedDir)):
        shutil.rmtree(exportedDir,1)
    if(debug):
        print "Try to find META-INF/monResources.xml"
    os.mkdir(exportedDir)
    AdminApp.exportFile(application,'META-INF/monResources.xml',os.path.join(exportedDir,"monResources.xml"))  
    exist = os.path.exists(os.path.join(exportedDir,"monResources.xml"))      
    if(debug):
        print "monResources.xml eixst? : ",exist
    shutil.rmtree(exportedDir,1)
    if(debug):
        print "Temporary folder " + exportedDir + " is cleaned."
    return exist
'''  
def isMonitorModel(application):
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isMonitorModel',"INFO: " +"Check if application '" + application + "' is monitor model application." )
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isMonitorModel',"INFO: " +"Try to find META-INF/monResources.xml")
    appMgr = AppManagementProxy.getLocalProxy()
    monResources = appMgr.getApplicationContents(application,'META-INF/monResources.xml',None,None)
    return monResources

def exportMonitorModels(snapshotDir,clusterName,nodeName,serverName):
    exported='false'
    applications = wsadminToList(AdminApp.list())    
    for application in applications:
        if(isMonitorModel(application) and isSpecifiedScope(application,clusterName,nodeName,serverName) == 'true'):
            print application + " is monitor model, will export it"
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'exportMonitorModels',"INFO: " +application + " is monitor model, will export it")
            AdminApp.export(application,snapshotDir + "/" + application + ".ear")
            exported = 'true'
            print "Montor model application " + application + " was exported to folder " + snapshotDir
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'exportMonitorModels',"INFO: " +"Montor model application " + application + " was exported to folder " + snapshotDir)
        else:
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'exportMonitorModels',"INFO: " +application + " is not monitor model.Skip it.")
    if(exported == 'false'):
        print "No monitor model were exported."
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'exportMonitorModels',"INFO: " +"No monitor model were exported.")
            
def exportFileRegistry(snapshotDir):  
    print "Start to export fileRegistry.xml"        
    #D187023 os.environ.get('CONFIG_ROOT') and os.environ.get('WAS_CELL') not work in Windows platform
    #Switch to use java.lang.System.getProperty
    configRootFolder = System.getProperty('was.repository.root')    
    cellName = System.getProperty('local.cell')
    cellRootFolder = os.path.join(os.path.join(configRootFolder, 'cells'),cellName)
    fileRegistryFile = os.path.join(cellRootFolder, 'fileRegistry.xml')
    extractedConfigPath = os.path.join(snapshotDir, 'config')
    if(not os.path.exists(extractedConfigPath)):
        os.makedirs(extractedConfigPath)
    targetFile = os.path.join(extractedConfigPath,'fileRegistry.xml')
    exist = os.path.exists(fileRegistryFile)
    if(exist):
        print fileRegistryFile + " was found, copy it to " + targetFile
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'exportFileRegistry',"INFO: " +fileRegistryFile + " was found, copy it to " + targetFile)
        shutil.copyfile(fileRegistryFile, targetFile)
    else:
        print fileRegistryFile + " not exits"
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'exportFileRegistry',"INFO: " +fileRegistryFile + " not exits")
        

def mergeFileRegistry(snapshotDir):
    from com.ibm.bpm.migration.configextraction import MergeFileRegistry
    from java.io import File
    mergeFileRegistryHelper = MergeFileRegistry()
    sourceFileRegistry = os.path.join(snapshotDir, 'config/fileRegistry.xml')    
    configRootFolder = System.getProperty('was.repository.root')    
    cellName = System.getProperty('local.cell')
    cellRootFolder = os.path.join(os.path.join(configRootFolder, 'cells'),cellName)
    targetUserRegistryFile = os.path.join(cellRootFolder, 'fileRegistry.xml')
    bakTargetUserRegistryFile = os.path.join(cellRootFolder, 'fileRegistry.xml.bak')
    exist = os.path.exists(targetUserRegistryFile)
    if(exist):
        if(not os.path.exists(bakTargetUserRegistryFile)):
            print "Backup " + targetUserRegistryFile + " to " + bakTargetUserRegistryFile
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'mergeFileRegistry',"INFO: Backup " + targetUserRegistryFile + " to " + bakTargetUserRegistryFile)
            shutil.copyfile(targetUserRegistryFile, bakTargetUserRegistryFile)
        else:
            print "fileRegistry file was already backup to " + bakTargetUserRegistryFile
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'mergeFileRegistry',"INFO: fileRegistry file was already backup to " + bakTargetUserRegistryFile)
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'mergeFileRegistry',"INFO: " +"Start to merge  " + sourceFileRegistry + " into " + targetUserRegistryFile)
        mergeFileRegistryHelper.merge(File(sourceFileRegistry), File(targetUserRegistryFile))        
    else:
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'mergeFileRegistry',"INFO: " +targetUserRegistryFile + " not found, will skip merge.")
        
def deployMonitorModels(snapshotDir,clusterName,nodeName,serverName):    
    from com.ibm.wbimonitor.util import ProductVersion
    from com.ibm.etools.commonarchive import EARFile
    from com.ibm.wbimonitor.monresources import MonitorWasResources
    from com.ibm.wbimonitor.lifecycle.spi import MMApplicationUtils
    modelApplications = [f for f in os.listdir(snapshotDir) if os.path.isfile(os.path.join(snapshotDir,f))]
    for applicationName in modelApplications:    
        if(not applicationName.endswith('.ear')):
            continue    
        applicationNameWithOutExt = applicationName[0:-4]
        print "Found monitor model application ear:" + applicationName    
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: " +"Found model application ear:" + applicationName)
        if(isApplicationInstalled(applicationNameWithOutExt) == 'False'):                    
            appEar = os.path.join(snapshotDir,applicationName)
            ear=EarUtils.getEarFile(appEar,0,1)
            mmResource=MMApplicationUtils.getMonitorWasResources(ear)
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: " +"Model general version:"+mmResource.getGeneratorVersion()) 
            mmVersion=ProductVersion(mmResource.getGeneratorVersion())
            eventSourceParam=''
            if (mmVersion.is61x()):
                print "v6.1.x Monitor Model was not supported, will skip it."
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: v6.1.x Monitor Model was not supported, will skip it.")
                return
            if (mmVersion.is62x() or mmVersion.is70x()):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: Detected that the model is either a 62x, 70x model. Setting ModelScopedIncomingEvents for the transport type.")
                eventSourceParam = ' -LifecycleClientEventSourceTask [[ ModelScopedIncomingEvents "" Inactive ]]'
            elif (mmVersion.is75x() or mmVersion.is80x()):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: Detected that the model is either a 75x, 80x model. Setting VersionScopedIncomingEvents for the transport type.")
                eventSourceParam = ' -LifecycleClientEventSourceTask [[ VersionScopedIncomingEvents "" Inactive ]]'
            else:
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: Detected that the model is either a 85x or higher model. Setting VersionScopedIncomingEvents for the transport type.")
                eventSourceParam = ' -LifecycleClientEventSourceTask [[ VersionScopedIncomingEvents "" Inactive ]]'
            if(clusterName):
                print "Deploy monitor model aplication " + applicationNameWithOutExt + " to cluster " + clusterName
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: " +"Deploy monitor model aplication " + applicationNameWithOutExt + " to cluster " + clusterName)
                AdminApp.install(appEar,'[-blaname "WebSphere:blaname=' + applicationNameWithOutExt + '" -nodeployejb -appname ' + applicationNameWithOutExt + ' -createMBeansForResources -noreloadEnabled -custom enhancedEarDisableValidation=true  -cluster ' + clusterName + eventSourceParam + ']')
            if(nodeName and serverName):
                print "Deploy monitor model aplication " + applicationNameWithOutExt + "to node " + nodeName + " server " + serverName
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: " +"Deploy monitor model aplication " + applicationNameWithOutExt + "to node " + nodeName + " server " + serverName)
                try:
                    AdminApp.install(appEar,'[-blaname "WebSphere:blaname=' + applicationNameWithOutExt + '" -nodeployejb -appname ' + applicationNameWithOutExt + ' -createMBeansForResources -noreloadEnabled -custom enhancedEarDisableValidation=true  -node ' + nodeName + ' -server ' + serverName + eventSourceParam +' ]')
                except Exception,e:
                    print "Some exception happened during monitor model application installation.Application is " +  appEar + " Exception:"+str(e)
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"ERROR: Some exceptio happened during monitor model installation, exception:"+str(e))
            #dep =AdminConfig.getid("/Deployment:" + applicationNameWithOutExt + "/")
            #deployedObject = AdminConfig.showAttribute(dep, "deployedObject")
            #AdminConfig.modify(deployedObject, [['startingWeight', '100']])
            AdminConfig.save()     
            print "Monitor model application " + applicationNameWithOutExt + " was migrated successfully."
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: " +"Monitor model application " + applicationNameWithOutExt + " was migrated successfully.")   
        else:
            print "Monitor model application " + applicationNameWithOutExt + " was already installed."    
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'deployMonitorModels',"INFO: " + "Monitor model application " + applicationNameWithOutExt + " was already installed." )
        updateMappedResourceForModelApplication(applicationNameWithOutExt)     
        
def isApplicationInstalled(applicationName):    
    isInstalled = "False"
    try:
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isApplicationInstalled',"INFO: " +"Check if application '" + applicationName + "' already existed ?")
        applications  = wsadminToList(AdminApp.list());        
        for application in applications:
            if (application == applicationName):
                isInstalled = "True";
                break;                
    except:
        print "Cannot check the application '" + applicationName + "' The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value          
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isApplicationInstalled',"ERROR: Cannot check the application '" + applicationName + "' The exception type is: " + str(sys.exc_type) + ". The exception description is: "+ str(sys.exc_value))
    return isInstalled;
#endDef

def updateMappedResourceForModelApplication(applicationName):
    resourceRefOption = getResourceRefOptionOfModeratorByJNDI(applicationName,'-MapResRefToEJB',9,'jdbc/RoutingDatabase')    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'updateMappedResourceForModelApplication',"INFO: " +"Updated MapResRefToEJB option of application " + applicationName + " is :" + str(resourceRefOption))
    options = []
    if(len(resourceRefOption) > 0):
        options += ["-MapResRefToEJB", resourceRefOption]            
        AdminApp.edit(applicationName, options)
        AdminConfig.save()    
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'updateMappedResourceForModelApplication','INFO: Target resource reference of jdbc/RoutingDatabase was updated to jdbc/wbm/MonitorDatabase')
    else:
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'updateMappedResourceForModelApplication','INFO: No need to update MapResRefToEJB option for application ' + applicationName)


def getResourceRefOptionOfModeratorByJNDI(appName,optionName,blockSize,jndiName):
    import AdminUtilities
    appDepRawContent = AdminUtilities.convertToList(AdminApp.view(appName, optionName))      
    appDepOptions = []

    if appDepRawContent:
      # the key/value pairs are separated by colons, and the keys may be translated    
      pattern = re.compile("([^:]*):\s*(.*)")
      # iterate all blocks (role mappings)
      for i in range(divmod(len(appDepRawContent), blockSize)[1], len(appDepRawContent), blockSize):
        # build the options list for the current block (role mapping)
        appDepOption = []
        found = 'false'
        for line in appDepRawContent[i:i + blockSize]:
          #print line
          matched = pattern.match(line)
          if matched:              
              if(found == 'true' and matched.group(1).strip() == 'Target Resource JNDI Name' and matched.group(2).strip() != 'jdbc/wbm/MonitorDatabase'):
                  #print "Test here"
                  appDepOption += ['jdbc/wbm/MonitorDatabase']
              elif(matched.group(1).strip() == 'Resource Reference' and matched.group(2).strip() == jndiName):
                  found = 'true'
                  appDepOption += [matched.group(2)]
              else:
                  appDepOption += [matched.group(2)]
              #print 'appDepOption:' + str(appDepOption)
        #endfor
        if(found == 'true'):
            appDepOptions += [appDepOption]
            break
      #endfor
    #endif

    return appDepOptions
#enddef
def backupDSSServiceConfigByModelID(backupModelFolderPath,modelID):    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelID',"INFO: Backup service level configuration of Scheduler Service for monitor model -- " + modelID)
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import DataServicesSchedulerMBeanFactory
    #from com.ibm.wbimonitor.lifecycle.spi.mbeans import DMSSchedulerMBeanFactory    
    #dmsSchedulerMBean = DMSSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )
    dssSchedulerMBean = DataServicesSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )
    configFilePath = os.path.join(backupModelFolderPath, 'DSSServiceConfig.properties')
    if(not os.path.exists(configFilePath)):
        configFile = open(configFilePath,'w')    
        dssBeansInfo = dssSchedulerMBean.getScheduledServicesInfoByModel(modelID)
        if(dssBeansInfo):
            configFile.write('###Service level DSS config of model -- ' + modelID + '###\n')
            for dssBean in dssBeansInfo:
                if(dssBean):       
                    #Write to the file with format SERVICE_NAME=Priority,Interval,IntervalUnit,StartTime,Status                          
                    configFile.write(dssBean.getServiceName() + '=' + str(dssBean.getServicePriority()) + ','+ str(dssBean.getServiceInterval()) +','+ str(dssBean.getIntervalUnit())+','+ str(dssBean.getStartTime()) +','+ str(dssBean.getStatus()) + '\n')                    
        else:                  
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelID',"INFO: No service level configuration of scheduler service for monitor model " + modelID)
        configFile.close()   
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelID','INFO: Service level configuration of Scheduler Service for monitor model -- ' + modelID + ' was persisted into file '+ configFilePath)    
    else:
        print 'Service level configuration of scheduler service for monitor model ' + modelID +' was already backup.'
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelID','INFO: Service level configuration of scheduler service for monitor model ' + modelID +' was already backup.')    
        
def backupDSSServiceConfigByModelVersion(backupModelFolderPath,modelID,versionList):    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelVersion',"INFO: " +"Backup version level configuration of Scheduler Service for monitor model by version -- " + modelID)
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import DataServicesSchedulerMBeanFactory
    #from com.ibm.wbimonitor.lifecycle.spi.mbeans import DMSSchedulerMBeanFactory    
    #dmsSchedulerMBean = DMSSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )
    dssSchedulerMBean = DataServicesSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )
    configFilePath = os.path.join(backupModelFolderPath, 'DSSServiceVersionConfig.properties')
    if(not os.path.exists(configFilePath)):
        configFile = open(configFilePath,'w')    
        configFile.write('###DSS version lelve config of model -- ' + modelID + ' by version ###\n')
        for version in versionList:
            if(version):
                for serviceName in serviceNameList:                    
                    versionNumber = version.getModelVersionId().getModelVersion()
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelVersion','INFO:Backup version level configuration of Scheduler Service for model id:' + modelID + 'version:' + str(versionNumber) + ' serviceName:'+ serviceName)
                    dssBeansInfo = dssSchedulerMBean.getSchedulerServiceVersionInfoByModelByServiceByVersion(modelID,serviceName,versionNumber)                                   
                    if(dssBeansInfo):                
                        #print "dssBeansInfo.getstatus()",dssBeansInfo.getStatus()        
                        #Write to the file with format SERVICE_NAME=Priority,Interval,IntervalUnit,StartTime,LastCompletedTime,Status,LastRunStatus                          
                        configFile.write(str(versionNumber) + '.' +serviceName +'=' + str(dssBeansInfo.getStatus()) + '\n') 
                        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigvi ration', 'backupDSSServiceConfigByModelVersion',"INFO: Version level configuration of scheduler service--" + serviceName +" for monitor model " + modelID + " with version "+str(versionNumber) + " was persisted." )
                    else:                  
                        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigvi ration', 'backupDSSServiceConfigByModelVersion',"INFO: No version level configuration of scheduler service--" + serviceName+" for monitor model " + modelID + " with version "+str(versionNumber) )
        configFile.close()   
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelVersion','INFO: Version level configuration of Scheduler Service for monitor model -- ' + modelID + ' was persisted into file '+ configFilePath)    
    else:
        print 'Version level configuration of scheduler service for monitor model ' + modelID +' was already backup.'
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSServiceConfigByModelVersion','INFO: Version level configuration of scheduler service for monitor model ' + modelID +' was already backup.')    
 
def backupDSSModelConfigByModelID(backupModelFolderPath,modelID):    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSModelConfigByModelID',"INFO: " +"Backup model level configuration of Scheduler Service for monitor model -- " + modelID)
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import DataServicesSchedulerMBeanFactory
    #from com.ibm.wbimonitor.lifecycle.spi.mbeans import DMSSchedulerMBeanFactory    
    #dmsSchedulerMBean = DMSSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )
    dssSchedulerMBean = DataServicesSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )
    configFilePath = os.path.join(backupModelFolderPath, 'DSSModelConfig.properties')
    if(not os.path.exists(configFilePath)):
        configFile = open(configFilePath,'w')    
        dssBeansInfo = dssSchedulerMBean.getSchedulerModelInfoByModel(modelID)
        if(dssBeansInfo):
            configFile.write('###DSS model level config of model -- ' + modelID + '###\n')
            configFile.write(modelID + '=' + str(dssBeansInfo.getPriority())+',' +str(dssBeansInfo.getStatus()) +'\n')                    
        else:                  
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSModelConfigByModelID',"INFO: " +"No model level configuration of scheduler service for monitor model " + modelID)
        configFile.close()   
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSModelConfigByModelID','INFO: Model level configuration of Scheduler Service for monitor model -- ' + modelID + ' was persisted into file '+ configFilePath)    
    else:
        print 'Model level configuration of scheduler service for monitor model ' + modelID +' was already backup.'
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSModelConfigByModelID','INFO: Model level configuration of scheduler service for monitor model ' + modelID +' was already backup.')    
       
def backupDSSConfigForModels(snapshotDir): 
    print "Start to backup configuration of scheduler service for all monitor model."
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSConfigForModels',"INFO: Start to backup configuration of scheduler service for all monitor model.")
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import LifecycleServicesMBeanFactory
    lifecysleMBean = LifecycleServicesMBeanFactory.getMBean( AdminControl.getAdminClient() )    
    models = lifecysleMBean.listModels()
    if(models):
        backupFolderPath = os.path.join(snapshotDir,'backup')
        if(not os.path.exists(backupFolderPath)):
            os.makedirs(backupFolderPath)
        for model in models:
            if(model):
                modelID = model.getModelID()
                backupModelFolderPath = os.path.join(backupFolderPath,modelID)
                if(not os.path.exists(backupModelFolderPath)):
                    os.makedirs(backupModelFolderPath)
                backupDSSModelConfigByModelID(backupModelFolderPath,modelID)
                backupDSSServiceConfigByModelID(backupModelFolderPath,modelID)
                versionList = lifecysleMBean.listModelVersions(modelID)
                if(versionList):
                    backupDSSServiceConfigByModelVersion(backupModelFolderPath,modelID,versionList)
    print "Backup configuration of scheduler service of for all monitor model was completed."
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSConfigForModels',"INFO: Backup configuration of scheduler service of for all monitor model was completed.")
    

def restoreDSSModelConfigByModelID(modelDSSConfigPath,modelID):
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Restore model level configuration of Scheduler Service for monitor model -- ' + modelID)   
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import DataServicesSchedulerMBeanFactory     
    from java.util import Properties
    from java.io import FileInputStream
    from java.text import SimpleDateFormat
    from java.util import Date
    from java.util import Locale
    from java.util import TimeZone
    configProps = Properties()    
    configProps.load(FileInputStream(modelDSSConfigPath))
    dssSchedulerMBean = DataServicesSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )    
    dssBeansInfo = dssSchedulerMBean.getScheduledServicesInfoByModel(modelID)
    count = 10
    while((not dssBeansInfo) and count > 0):
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Can not get scheduler service configuration for model ' + modelID + ' that may be caused by model deployment was not ready yet, will sleep 10 seconds and try again.')
        time.sleep(10)
        dssBeansInfo = dssSchedulerMBean.getScheduledServicesInfoByModel(modelID)
        count = count - 1
    if(not dssBeansInfo):
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Can not get scheduler service configuration for model ' + modelID + ' that may be caused by model deployment was not ready yet, please return command MonitorModelMigration later again.')
        return 'false'
    else:
        value = configProps.getProperty(modelID)
        count = 5
        if(value):        
            props = value.split(',')
            dssSchedulerMBean.setModelPriorityByModel(modelID,long(props[0]))    
            status = props[1]
            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Priority of monitor model ' + modelID + ' :' +props[0])
            result = 'true'  
            if(status == '0'):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Status of monitor model ' + modelID +':' + str(status) + ', will resume model.')
                dssSchedulerMBean.resumeModel(modelID)
                result = isActionSuccess(modelID, None,None,0)
                while(result != 'true' and count > 0):
                    time.sleep(10)
                    dssSchedulerMBean.suspendModel(modelID)
                    result = isActionSuccess(modelID, None,None,0)
                    count = count - 1                
            if(status == '1'):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Status of monitor model ' + modelID +':' + str(status) + ', will suspend model.')
                dssSchedulerMBean.suspendModel(modelID)
                result = isActionSuccess(modelID, None,None,1)
                while(result != 'true'  and count > 0):
                    time.sleep(10)
                    dssSchedulerMBean.suspendModel(modelID)
                    result = isActionSuccess(modelID, None,None,1)
                    count = count - 1
            if(result == 'true'):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Model level configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored from file '+ modelDSSConfigPath)
            else:
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSModelConfigByModelID','INFO: Failed to restore model level configuration of Scheduler Service for monitor model -- ' + modelID + '.Please return command MonitorModelMigration later again.')
            return result
        
def restoreDSSServiceConfigByModelID(modelDSSServiceConfigPath,modelID):
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelID','INFO: Restore service level configuration of Scheduler Service for monitor model -- ' + modelID)   
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import DataServicesSchedulerMBeanFactory     
    from java.util import Properties
    from java.io import FileInputStream
    from java.text import SimpleDateFormat
    from java.util import Date
    from java.util import Locale
    from java.util import TimeZone    
    configProps = Properties()    
    configProps.load(FileInputStream(modelDSSServiceConfigPath))
    dssSchedulerMBean = DataServicesSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )    
    for key in configProps.keySet():
        value = configProps.getProperty(str(key))
        if(value):            
            props = value.split(',')
            intervalUnit = ''
            if(props[2] == '1'):
                intervalUnit = 'MINUTE'
            if(props[2] == '2'):
                intervalUnit = 'HOUR'
            if(props[2] == '3'):
                intervalUnit = 'DAY'
            if(props[2] == '4'):
                intervalUnit = 'WEEK'
            if(props[2] == '5'):
                intervalUnit = 'MONTH'     
            #For of startTime in file is like 'Mon 2014-01-01 12:00:00.000', just leave '2014-01-01 12:00:00'    
            strStartTime = props[3][4:-4]    
            TimeZone.setDefault(TimeZone.getTimeZone('UTC'))
            dateFormat=SimpleDateFormat('yyyy-MM-dd HH:mm:ss',Locale.US)
            startTime = dateFormat.parse(strStartTime)
            count = 10
            dssBeansInfo = dssSchedulerMBean.getSchedulerServiceInfoByModelByService(modelID,str(key))
            while((not dssBeansInfo) and count > 0):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelID','INFO: Can not get scheduler service(' + str(key) +') configuration for model ' + modelID + ' that may be caused by model deployment was not ready yet, will sleep 10 seconds and try again.')
                time.sleep(10)
                dssBeansInfo = dssSchedulerMBean.getSchedulerServiceInfoByModelByService(modelID,str(key))
                count = count - 1
            if(not dssBeansInfo):
                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelID','INFO: Can not get scheduler service(' + str(key) +') configuration for model ' + modelID + ' that may be caused by model deployment was not ready yet, please return command MonitorModelMigration later again.')
                return 'false'
            else:        
                dssSchedulerMBean.setServiceIntervalByModelAndService(modelID,str(key),long(props[1]),intervalUnit,startTime)
                #if(long(props[0]) <= 50):
                #dssSchedulerMBean.setModelPriorityByModel(modelID,long(props[0]))
                #else:
                #    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Priority value of service ' + str(key) + 'for model '+modelID + ' is greater than 50, will not set it.')
                status = props[4]     
                count = 5   
                result = 'true'
                if(status == '0'):
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelID','INFO: Status of service ' +str(key)+ ' for monitor model ' + modelID + ':' + status + ', will resume service ' + str(key))
                    dssSchedulerMBean.resumeScheduledServiceByModelAndService(modelID,str(key))
                    result = isActionSuccess(modelID, str(key),None,0)
                    while(result != 'true' and count > 0):
                        time.sleep(10)
                        dssSchedulerMBean.resumeScheduledServiceByModelAndService(modelID,str(key))
                        result = isActionSuccess(modelID, str(key),None,0)
                        count = count - 1                    
                if(status == '1'):
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelID','INFO: Status of service ' +str(key)+ ' for monitor model ' + modelID + ':' + status + ', will suspend service ' + str(key))
                    dssSchedulerMBean.suspendScheduledServiceByModelAndService(modelID,str(key))
                    result = isActionSuccess(modelID, str(key),None,1)
                    while(result != 'true' and count > 0):
                        time.sleep(10)
                        dssSchedulerMBean.suspendScheduledServiceByModelAndService(modelID,str(key))
                        result = isActionSuccess(modelID, str(key),None,1)
                        count = count - 1
                if(result == 'true'):
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelID','INFO: Restore service level configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored from file ' + modelDSSServiceConfigPath)
                else:
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelID','INFO: Failed to restore service level configuration of Scheduler Service for monitor model -- ' + modelID + '.Please return command MonitorModelMigration later again.')
                    return result        
    return 'true'       


def restoreDSSServiceConfigByModelIDAndVersion(modelVersionDSSServiceConfigPath,modelID,versionList):
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Restore version level configuration of Scheduler Service for monitor model -- ' + modelID)   
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import DataServicesSchedulerMBeanFactory     
    from java.util import Properties
    from java.io import FileInputStream
    from java.text import SimpleDateFormat
    from java.util import Date
    from java.util import Locale
    from java.util import TimeZone    
    configProps = Properties()    
    configProps.load(FileInputStream(modelVersionDSSServiceConfigPath))
    dssSchedulerMBean = DataServicesSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )    
    if(versionList):
        for version in versionList:
            if(version):
                versionNumber = version.getModelVersionId().getModelVersion()
                for serviceName in serviceNameList:
                    status = configProps.getProperty(str(versionNumber) + '.' + serviceName)
                    if(status):        
                        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Status of monitor model ' + modelID + ' version--' +str(versionNumber) +' serviceName--' + serviceName +' :' +str(status))  
                        count = 10
                        dssBeansInfo = dssSchedulerMBean.getSchedulerServiceVersionInfoByModelByServiceByVersion(modelID,serviceName,versionNumber)
                        while((not dssBeansInfo) and count > 0):
                            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Can not get scheduler service(' + str(serviceName) +') configuration for model ' + modelID + ' version ' + str(versionNumber) + ' that may be caused by model deployment was not ready yet, will sleep 10 seconds and try again.')
                            time.sleep(10)
                            dssBeansInfo = dssSchedulerMBean.getSchedulerServiceVersionInfoByModelByServiceByVersion(modelID,serviceName,versionNumber)
                            count = count - 1
                        if(not dssBeansInfo):
                            log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Can not get scheduler service(' + str(serviceName) +') configuration for model ' + modelID + ' version ' + str(versionNumber) + ' that may be caused by model deployment was not ready yet, please return command MonitorModelMigration later again.')
                            return 'false'
                        else:  
                            result = 'true'
                            count = 5
                            if(status == '0'):    
                                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Status of service ' +serviceName+ ' for monitor model ' + modelID + ' and version ' + str(versionNumber) +':' + status + ', will resume service ' + serviceName)                        
                                dssSchedulerMBean.resumeScheduledServiceByModelAndServiceAndVersion(modelID,serviceName,versionNumber)
                                result = isActionSuccess(modelID, serviceName,versionNumber,0)
                                while(result != 'true' and count > 0):
                                    dssSchedulerMBean.resumeScheduledServiceByModelAndServiceAndVersion(modelID,serviceName,versionNumber)
                                    result = isActionSuccess(modelID, serviceName,versionNumber,0)
                                    count = count - 1                            
                            if(status == '1'):
                                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Status of service ' +serviceName+ ' for monitor model ' + modelID + ' and version ' + str(versionNumber) +':' + status + ', will suspend service ' + serviceName)
                                dssSchedulerMBean.suspendScheduledServiceByModelAndServiceAndVersion(modelID,serviceName,versionNumber)
                                result = isActionSuccess(modelID, serviceName,versionNumber,1)
                                while(result != 'true' and count > 0):
                                    dssSchedulerMBean.suspendScheduledServiceByModelAndServiceAndVersion(modelID,serviceName,versionNumber)
                                    result = isActionSuccess(modelID, serviceName,versionNumber,1)
                                    count = count - 1    
                            if(result == 'true'):
                                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Version level configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored from file '+ modelVersionDSSServiceConfigPath)
                            else:
                                log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSServiceConfigByModelIDAndVersion','INFO: Failed to restore version level configuration of Scheduler Service for monitor model -- ' + modelID + '.Please return command MonitorModelMigration later again.')
                                return result
    return 'true'                         
    
    
def restoreDSSConfigForModels(snapshotDir):
    print 'Start to restore configuration of Scheduler Service for all monitor model.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSConfigForModels','INFO: Start to restore configuration of Scheduler Service for all monitor model.')
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import LifecycleServicesMBeanFactory
    lifecysleMBean = LifecycleServicesMBeanFactory.getMBean( AdminControl.getAdminClient() )          
    models = lifecysleMBean.listModels()
    if(models):
        for model in models:
            if(model):
                modelID = model.getModelID()
                modelBackupFolderPath = os.path.join(snapshotDir,'backup/' + modelID)  
                result1 = 'false'
                result2 = 'false'
                result3 = 'false'
                if(os.path.exists(modelBackupFolderPath) and not os.path.exists(os.path.join(modelBackupFolderPath,'resotre.complete'))):
                    modelDSSConfigPath = os.path.join(modelBackupFolderPath,'DSSModelConfig.properties')
                    if (os.path.exists(modelDSSConfigPath) and os.path.isfile(modelDSSConfigPath)):            
                        result1 = restoreDSSModelConfigByModelID(modelDSSConfigPath,modelID)
                    modelDSSServiceConfigPath = os.path.join(modelBackupFolderPath,'DSSServiceConfig.properties')
                    if (os.path.exists(modelDSSServiceConfigPath) and os.path.isfile(modelDSSServiceConfigPath)):            
                        result2 = restoreDSSServiceConfigByModelID(modelDSSServiceConfigPath,modelID)                        
                    versionList = lifecysleMBean.listModelVersions(modelID)
                    modelVersionDSSServiceConfigPath = os.path.join(modelBackupFolderPath,'DSSServiceVersionConfig.properties')
                    if (os.path.exists(modelVersionDSSServiceConfigPath) and os.path.isfile(modelVersionDSSServiceConfigPath)):
                        result3 = restoreDSSServiceConfigByModelIDAndVersion(modelVersionDSSServiceConfigPath,modelID,versionList)
                    #log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSConfigForModels','INFO: Configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored from file '+ configFilePath)
                    if(result1 == 'true' and result2 == 'true' and result3 == 'true'):
                        print 'All configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored successfully.'
                        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSConfigForModels','INFO: All configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored successfully.')
                        completeFile = open(os.path.join(modelBackupFolderPath,'resotre.complete'),'w')
                        completeFile.write('done\n')
                        completeFile.close()
                    else:
                        print 'Not all configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored successfully.Please run MonitorModelMigration command later again.Please get more details from migration log.' 
                        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSConfigForModels','INFO: Not all configuration of Scheduler Service for monitor model -- ' + modelID + ' was restored successfully.Please run MonitorModelMigration command later again.')
                else:
                    print 'No backup of scheduler service configuratio for model ' + modelID + ' or restore of scheduler service configuration has been done, skip it.' 
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSConfigForModels','INFO: No backup of scheduler service configuratio for model ' + modelID + ' or restore of scheduler service configuration has been done, skip it.')
    print 'Restore configuration of Scheduler Service for all monitor model was completed.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreDSSConfigForModels','INFO: Restore configuration of Scheduler Service for all monitor model was completed.')

def isActionSuccess(modelID,serviceName,version,action):
    actionType =''
    if(action == 0):
        actionType = 'resume'
    if(action == 1):
        actionType = 'suspend'    
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import DataServicesSchedulerMBeanFactory    
    dssSchedulerMBean = DataServicesSchedulerMBeanFactory.getMBean( AdminControl.getAdminClient() )    
    status = -1
    if(modelID and serviceName and version):
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isActionSuccess','INFO: Check if ' + actionType +' action of service -- ' +serviceName + ' for model-- '+ modelID + ' version-- ' + str(version)+ ' was success.')
        dssBeansInfo = dssSchedulerMBean.getSchedulerServiceVersionInfoByModelByServiceByVersion(modelID,serviceName,version)
        if(dssBeansInfo):
            status = dssBeansInfo.getStatus()
    elif(modelID and serviceName):
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isActionSuccess','INFO: Check if ' + actionType +' action of service -- ' +serviceName + ' for model-- '+ modelID + ' was success.')
        dssBeansInfo = dssSchedulerMBean.getSchedulerServiceInfoByModelByService(modelID,serviceName)
        if(dssBeansInfo):
            status = dssBeansInfo.getStatus()
    elif(modelID):
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isActionSuccess','INFO: Check if ' + actionType +' action for model-- '+ modelID + ' was success.')
        dssBeansInfo = dssSchedulerMBean.getSchedulerModelInfoByModel(modelID)
        if(dssBeansInfo):
            status = dssBeansInfo.getStatus()
    if(status == action):
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isActionSuccess','INFO: ' + actionType +' action was success.')
        return 'true'
    else:
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'isActionSuccess','INFO: ' + actionType +' action was failed.Will retry.')
        return 'false'


def getBusinessSpaceDBName():
    import AdminUtilities
    dbName = ''
    dataSourceList = AdminConfig.list('DataSource')
    dataSources = AdminUtilities.convertToList(dataSourceList)
    for ds in dataSources:
        name=AdminConfig.showAttribute(ds,'name')
        if(name == 'mashupDS'):
            propSet = AdminConfig.showAttribute(ds,'propertySet')
            propList = AdminUtilities.convertToList(AdminConfig.list('J2EEResourceProperty',propSet))
            for propid in propList:
                propName = AdminConfig.showAttribute(propid,'name')
                if(propName == 'databaseName'):
                    dbName = AdminConfig.showAttribute(propid,'value')
    return dbName

def getBSpaceSchemaName():
    import AdminUtilities
    schemaName = ''
    dataSourceList = AdminConfig.list('ResourceEnvironmentProvider')
    dataSources = AdminUtilities.convertToList(dataSourceList)
    for ds in dataSources:
        name=AdminConfig.showAttribute(ds,'name')
        if(name == 'Mashups_ConfigService'):
            propSet = AdminConfig.showAttribute(ds,'propertySet')
            propList = AdminUtilities.convertToList(AdminConfig.list('J2EEResourceProperty',propSet))
            for propid in propList:
                propName = AdminConfig.showAttribute(propid,'name')
                if(propName == 'schemaName'):
                    schemaName = AdminConfig.showAttribute(propid,'value')
                    if (schemaName.endswith('.')):
                        schemaName = schemaName[0:-1]
    return schemaName

def getBSpaceDBType():
    import AdminUtilities
    dbType = ''
    dataSourceList = AdminConfig.list('DataSource')
    dataSources = AdminUtilities.convertToList(dataSourceList)
    for ds in dataSources:
        name=AdminConfig.showAttribute(ds,'name')
        if(name == 'mashupDS'):
            dsHelperClassName=AdminConfig.showAttribute(ds,'datasourceHelperClassname')
            if(dsHelperClassName.find('DB2') > 0):
                dbType = 'db2'
            if(dsHelperClassName.find('Oracle') > 0):
                dbType = 'oracle'
            if(dsHelperClassName.find('SQLServer') > 0):
                dbType = 'sqlserver'            
    return dbType

def generateMashupMigrationPropsFile(snapshotDir):        
    from java.util import Properties
    from java.io import File
    from java.io import FileInputStream
    from java.io import FileOutputStream   
    sourceVersion = '7.0.0.x'
    wasInstallRoot = System.getProperty('was.install.root')
    sourceVersionPropsFile = os.path.join(snapshotDir, 'config/sourceVersion.properties')
    sourceVersionProps = Properties()
    if(os.path.exists(sourceVersionPropsFile)):
        sourceVersionFis = FileInputStream(File(sourceVersionPropsFile))
        sourceVersionProps.load(sourceVersionFis)
        sourceVersion = sourceVersionProps.getProperty('source.version')
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'generateMashupMigrationPropsFile','INFO: Source Monitor product version is '+sourceVersion)
    if(not sourceVersion):
        sourceVersion = '7.0.0.x'
    propTemplateFilePath = os.path.join(wasInstallRoot, 'BusinessSpace/mm.runtime/migration/migration2002to3001.properties')    
    if(not os.path.exists(propTemplateFilePath)):
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'generateMashupMigrationPropsFile',"INFO: " + propTemplateFilePath + " was not found.There will be no migration2002to3001.properties generated.")
        return
    if(sourceVersion.startswith('7.0')):
        print 'Start to generate migration2002to3001.properties for Business Space data migration for source version v7.0.0.x.'
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'generateMashupMigrationPropsFile','INFO: Start to generate migration2002to3001.properties for Business Space data migration for source version v7.0.0.x.')
        bspaceDataMigrationProps = Properties()
        fis = FileInputStream(File(propTemplateFilePath))
        bspaceDataMigrationProps.load(fis)
        bspaceDataMigrationProps.put('source.datasource.name', 'mashupDS')
        bspaceDataMigrationProps.put('target.datasource.name', 'mashupDS')
        bspaceDataMigrationProps.put('source.db.schema.name', 'BSPMGTMP')
        sourceDBName = getBusinessSpaceDBName()
        targetSchemaName = getBSpaceSchemaName()
        targetDBType = getBSpaceDBType()
        bspaceDataMigrationProps.put('source.db.database.name',sourceDBName)
        bspaceDataMigrationProps.put('target.db.schema.name',targetSchemaName)
        bspaceDataMigrationProps.put('db.type',targetDBType)
        mashupPropsFilePath = os.path.join(snapshotDir, 'BusinessSpace/migration/migration2002to3001.properties')    
        if (not os.path.exists(os.path.join(snapshotDir, 'BusinessSpace/migration'))):
            os.makedirs(os.path.join(snapshotDir, 'BusinessSpace/migration'))
        bspaceDataMigrationProps.store(FileOutputStream(File(mashupPropsFilePath)), '')
        print mashupPropsFilePath + ' was generated.'
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'generateMashupMigrationPropsFile','INFO: '+mashupPropsFilePath + ' was generated.')
    else:
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'generateMashupMigrationPropsFile',"INFO: Source version is not v7.0.0.x. There will be no migration2002to3001.properties generated.")
    
def recordSourceProductVersion():
    from com.ibm.websphere.product import WASDirectory
    from com.ibm.websphere.product import WASProductInfo
    from java.util import Properties   
    from java.io import File   
    from java.io import FileOutputStream  
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'recordSourceProductVersion','INFO: Will record source Monitor product version into property file.')
    wasDirectory=WASDirectory(java.lang.System.getProperty('was.install.root'))
    info=wasDirectory.getWASProductInfo('WBM')
    if(info):
        version = info.getVersion()
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'recordSourceProductVersion','INFO: Source Monitor product version is ' + version)
        sourceVersionFile = os.path.join(snapshotDir, 'config/sourceVersion.properties')
        sourceVersionProps = Properties()    
        sourceVersionProps.put('source.version',version)
        sourceVersionProps.store(FileOutputStream(File(sourceVersionFile)), '')
        print 'Source product version was recorded into ' + sourceVersionFile
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'recordSourceProductVersion','INFO: Source product version was recorded into ' + sourceVersionFile)
    else:
        print 'Source Monitor product version was not found.'
        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'recordSourceProductVersion','INFO: Source Monitor product version was not found.')

def restoreSchemaDeletionOption(snapshotDir):
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreSchemaDeletionOption','INFO: Start to restore option of schema deletion for all monitor model.')
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import LifecycleServicesMBeanFactory
    from java.util import Properties
    from java.io import FileInputStream
    lifecysleMBean = LifecycleServicesMBeanFactory.getMBean( AdminControl.getAdminClient() )          
    models = lifecysleMBean.listModels()
    if(models):
        for model in models:
            if(model):
                modelID = model.getModelID()
                modelBackupFolderPath = os.path.join(snapshotDir,'backup/' + modelID)  
                modelSchemaDelOptionFile = os.path.join(modelBackupFolderPath,'schemaDelOption.properties')
                if(os.path.exists(modelBackupFolderPath) and os.path.exists(modelSchemaDelOptionFile)):
                    configProps = Properties()    
                    configProps.load(FileInputStream(modelSchemaDelOptionFile))
                    for propName in configProps.keySet():
                        propValue = configProps.getProperty(str(propName))
                        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupSchemaDeletionOption',"INFO: Will restore schema deletion option for model " + modelID + " version " + str(propName))
                        deploymentProps=lifecysleMBean.readDeploymentOptions(modelID,long(propName))
                        deploymentProps.setProperty('com.ibm.wbimonitor.lifecycle.deploy.DeleteSchema',propValue)
                        lifecysleMBean.applyDeploymentOptions(modelID,long(propName),deploymentProps)
                        log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupSchemaDeletionOption',"INFO: Schema deletion option for model " + modelID + " version " + str(propName) + " was restored.It was set to be " + propValue)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'restoreSchemaDeletionOption','INFO: Restore option of schema deletion for all monitor model was completed.')
def backupSchemaDeletionOptionOfModels(snapshotDir):
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupSchemaDeletionOption',"INFO: " +"Start to backup option of schema deletion for all monitor model.")
    from com.ibm.wbimonitor.lifecycle.spi.mbeans import LifecycleServicesMBeanFactory
    lifecysleMBean = LifecycleServicesMBeanFactory.getMBean( AdminControl.getAdminClient() )    
    models = lifecysleMBean.listModels()
    if(models):
        backupFolderPath = os.path.join(snapshotDir,'backup')
        if(not os.path.exists(backupFolderPath)):
            os.makedirs(backupFolderPath)
        for model in models:
            if(model):
                modelID = model.getModelID()
                backupModelFolderPath = os.path.join(backupFolderPath,modelID)
                if(not os.path.exists(backupModelFolderPath)):
                    os.makedirs(backupModelFolderPath)      
                configFilePath = os.path.join(backupModelFolderPath, 'schemaDelOption.properties')
                if(not os.path.exists(configFilePath)):
                    configFile = open(configFilePath,'w')           
                    versionList = lifecysleMBean.listModelVersions(modelID)
                    if(versionList):
                        for version in versionList:
                            if(version):
                                versionNumber = version.getModelVersionId().getModelVersion()
                                deploymentProps=lifecysleMBean.readDeploymentOptions(modelID,versionNumber)
                                option = deploymentProps.getProperty('com.ibm.wbimonitor.lifecycle.deploy.DeleteSchema')
                                if(option):
                                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupSchemaDeletionOption',"INFO: Schema deletion option of model " + modelID + " version " + str(versionNumber) +" is " + option)
                                    configFile.write(str(versionNumber)+'='+option)
                                else:
                                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupSchemaDeletionOption',"INFO: Schema deletion option of model " + modelID + " version " + str(versionNumber) +" is not found.")
                    configFile.close()   
                else:
                    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupSchemaDeletionOption',"INFO: Schema deletion option of models was already backup.")
                                         
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'backupDSSConfigForModels',"INFO: " +"Backup option of schema deletion for all monitor model was completed.")
    
    
def printUsage():
    print "Parameter is not correct.Usage:\n\t"
    print "For extracting monitor model from source environment:"
    print "<DmgrProfile>/bin/wadmin -conntype NONE -wsadmin_classpath <Source_Monitor_Install>/lib/wbminstall_log.jar -f MonitorModelMigration.py -extract<Path_Of_Flder_ExtractedModels> <clusterName> | <nodeName> <serverName>\n"
    print "Example: \n\t<DmgrProfile>/bin/wadmin -conntype NONE -wsadmin_classpath /opt/Monitor751/lib/wbminstall_log.jar -f MonitorModelMigration.py -extract /tmp/extractedModels SingleCluster \n\t"   
    sys.exit(-1)

#------------------------------------------------------------------------------
#main start
#------------------------------------------------------------------------------

backupAndRestoreDSSConfig='true'  ### If we need keep configuration monitor scheduler service, by default yes
backupSchemaDeletionOption='true'  ### If we need keep option of deleting schema while uninstalling monitor model
snapshotDir=""
clusterName=""
nodeName=""
serverName=""
if(len(sys.argv) < 3 or len(sys.argv) > 4):
    printUsage()
if( sys.argv[0] != "-extract" and sys.argv[0] != "-migrate" ):
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'ERROR: Parameter is not correct,please check and try again.')    
    printUsage()

if(sys.argv[0] == "-extract" and len(sys.argv) == 3):    
    snapshotDir=sys.argv[1]
    clusterName=sys.argv[2]     
    print 'Start to extract monitor models, migration logs will be saved into ' + os.path.join(logDir,logName)    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to extract monitor models deployed on cluster '+ clusterName +', model will be saved to ' + snapshotDir)
    exportMonitorModels(snapshotDir,clusterName,None,None)
    print 'Extract of monitor models was completed.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Extract of monitor models was completed.')
    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to extract fileRegistry.xml.')
    exportFileRegistry(snapshotDir)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Extract of fileRegistry.xml was completed.')
    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to record source monitor product.')
    recordSourceProductVersion()
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Record of source monitor product version was completed.')    
    
if(sys.argv[0] == "-extract" and len(sys.argv) == 4):
    snapshotDir=sys.argv[1]
    nodeName=sys.argv[2]     
    serverName=sys.argv[3]
    print 'Start to extract monitor models, migration logs will be saved into ' + os.path.join(logDir,logName)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to extract monitor models deployed on server '+ serverName +', model will be saved to ' + snapshotDir)
    exportMonitorModels(snapshotDir,None,nodeName,serverName)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Extract of monitor models was completed.')
    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to extract fileRegistry.xml.')
    exportFileRegistry(snapshotDir)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Extract of fileRegistry.xml was completed.')
    
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to record source monitor product.')
    recordSourceProductVersion()
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Record of source monitor product version was completed.')    

if(sys.argv[0] == "-migrate" and len(sys.argv) == 3):
    snapshotDir=sys.argv[1]
    clusterName=sys.argv[2]    
    print 'Start to migrate the monitor model into target monitor environment, migration logs will be saved into ' + os.path.join(logDir,logName)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Start to migrate the monitor model into target monitor environment.')
             
    if(backupAndRestoreDSSConfig == 'true'):
        backupDSSConfigForModels(snapshotDir)
    
    if(backupSchemaDeletionOption == 'true'):
        backupSchemaDeletionOptionOfModels(snapshotDir)   
    
    deployMonitorModels(snapshotDir,clusterName,None,None)     
   
    print 'Start to merge fileRegistry.xml into target Monitor environment.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to merge fileRegistry.xml into target Monitor environment.')
    mergeFileRegistry(snapshotDir)   
    print 'Merge of fileRegistry.xml was completed.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Merge of fileRegistry.xml was completed.')   
    
    if(backupAndRestoreDSSConfig == 'true'):
        restoreDSSConfigForModels(snapshotDir)    
    
    if(backupSchemaDeletionOption == 'true'):
        restoreSchemaDeletionOption(snapshotDir)
        
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Start to generate Mashup datasource property file for Business Space data migration.')
    generateMashupMigrationPropsFile(snapshotDir)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Generation of Mashup datasource property file was completed.')
    
    print 'Migration of monitor model was completed.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Migration of monitor model was completed.')    
   
if(sys.argv[0] == "-migrate" and len(sys.argv) == 4):
    snapshotDir=sys.argv[1]
    nodeName=sys.argv[2]     
    serverName=sys.argv[3]
    print 'Start to migrate the monitor model into target monitor environment, migration logs will be saved into ' + os.path.join(logDir,logName)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Start to migrate the monitor model into target monitor environment.')
          
    if(backupAndRestoreDSSConfig == 'true'):
        backupDSSConfigForModels(snapshotDir) 
    
    if(backupSchemaDeletionOption == 'true'):
        backupSchemaDeletionOptionOfModels(snapshotDir)
    
    deployMonitorModels(snapshotDir,None,nodeName,serverName)    
   
    print 'Start to merge fileRegistry.xml into target Monitor environment.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Start to merge fileRegistry.xml into target Monitor environment.')
    mergeFileRegistry(snapshotDir)   
    print 'Merge of fileRegistry.xml was completed.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main', 'INFO: Merge of fileRegistry.xml was completed.') 
    
    if(backupAndRestoreDSSConfig == 'true'):
        restoreDSSConfigForModels(snapshotDir)    
    
    if(backupSchemaDeletionOption == 'true'):
        restoreSchemaDeletionOption(snapshotDir)
        
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Start to generate Mashup datasource property file for Business Space data migration.')
    generateMashupMigrationPropsFile(snapshotDir)
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Generation of Mashup datasource property file was completed.')

    print 'Migration of monitor model was completed.'
    log.msg(ILogger.TYPE_INFO, 'MonitorModelMigration', 'main','INFO: Migration of monitor model was completed.')  
